// DevChange.pas contains the consts and records to catch the
// WM_DEVICECHANGE event of Windows.
//
// to really catch the event declare and implement a function in your TForm class
// procedure WMDeviceChange(var msg: TWMDeviceChange); message WM_DEVICECHANGE;

unit DevChange;

interface

uses
   Windows; // for DWORD etc.

const
   // the Event values
   DBT_DEVNODES_CHANGED             = $0007;
   DBT_QUERYCHANGECONFIG            = $0017;
   DBT_CONFIGCHANGED                = $0018;
   DBT_CONFIGCHANGECANCELED         = $0019;
   DBT_MONITORCHANGE                = $001B;
   DBT_SHELLLOGGEDON                = $0020;
   DBT_CONFIGMGAPI32                = $0022;
   DBT_VXDINITCOMPLETE              = $0023;
   DBT_NO_DISK_SPACE                = $0047;
   DBT_LOW_DISK_SPACE               = $0048;

   DBT_DEVICEARRIVAL                = $8000;  // system detected a new device
   DBT_DEVICEQUERYREMOVE            = $8001;  // wants to remove, may fail
   DBT_DEVICEQUERYREMOVEFAILED      = $8002;  // removal aborted
   DBT_DEVICEREMOVEPENDING          = $8003;  // about to remove, still avail.
   DBT_DEVICEREMOVECOMPLETE         = $8004;  // device is gone
   DBT_DEVICETYPESPECIFIC           = $8005;  // type specific event
   DBT_CUSTOMEVENT                  = $8006;  // user-defined event

   // the DevType values
   DBT_DEVTYP_OEM                   = $00000000;  // oem-defined device type
   DBT_DEVTYP_DEVNODE               = $00000001;  // devnode number
   DBT_DEVTYP_VOLUME                = $00000002;  // logical volume
   DBT_DEVTYP_PORT                  = $00000003;  // serial, parallel
   DBT_DEVTYP_NET                   = $00000004;  // network resource
   DBT_DEVTYP_DEVICEINTERFACE       = $00000005;  // device interface class
   DBT_DEVTYP_HANDLE                = $00000006;  // file system handle

type
   // the event structure for WM_DEVICECHANGE
   // just a variant of all the TWM* structures of Delphi
   TWMDeviceChange = record
      Msg   : Cardinal;   // WM_DEVICECHANGE
      Event : Integer;    // DBT_*
      Device: PChar;      // points to a TDevBroadcastHeader
      Result: Longint;    // no need to use this
   end;

   // this is the common part of all the structures you can get
   // typecast to the correct one by inspecting DevType and Size
   TDevBroadcastHeader = record
       Size    : DWORD;  // size of this structure
       DevType : DWORD;  // DBT_DEVTYP_*
       Reserved: DWORD;  // unknown
   end;

   // this is the structure you get with DevType = DBT_DEVTYP_DEVNODE
   TDevBroadcastDevnode = record
       Size    : DWORD;  // size of this structure
       DevType : DWORD;  // DBT_DEVTYP_DEVNODE
       Reserved: DWORD;  // unknown (seems unused here)
       DevID   : DWORD;  // not a pointer but a registry key
   end;

   // pointer types Microsoft style
   PTDevBroadcastHeader  = ^TDevBroadcastHeader;
   PTDevBroadcastDevnode = ^TDevBroadcastDevnode;

implementation

end.
