// Hid.pas contains all the consts, types and prototypes to use
// the functions of HID.DLL.

unit Hid;

interface

uses
   Windows,   // for DWORD etc
   HidUsage;  // for TUsage

const
   // internal constant to name the DLL in prototypes
   hiddll = 'hid.dll';

   // Define NT Status codes with Facility Code of FACILITY_HID_ERROR_CODE
   // these are the return values of all the HidP* functions
   cHIDPStatus_SUCCESS                  = $00110000;
   cHIDPStatus_NULL                     = $80110001;
   cHIDPStatus_INVALID_PREPARSED_DATA   = $C0110001;
   cHIDPStatus_INVALID_REPORT_TYPE      = $C0110002;
   cHIDPStatus_INVALID_REPORT_LENGTH    = $C0110003;
   cHIDPStatus_USAGE_NOT_FOUND          = $C0110004;
   cHIDPStatus_VALUE_OUT_OF_RANGE       = $C0110005;
   cHIDPStatus_BAD_LOG_PHY_VALUES       = $C0110006;
   cHIDPStatus_BUFFER_TOO_SMALL         = $C0110007;
   cHIDPStatus_INTERNAL_ERROR           = $C0110008;
   cHIDPStatus_I8242_TRANS_UNKNOWN      = $C0110009;
   cHIDPStatus_INCOMPATIBLE_REPORT_ID   = $C011000A;
   cHIDPStatus_NOT_VALUE_ARRAY          = $C011000B;
   cHIDPStatus_IS_VALUE_ARRAY           = $C011000C;
   cHIDPStatus_DATA_INDEX_NOT_FOUND     = $C011000D;
   cHIDPStatus_DATA_INDEX_OUT_OF_RANGE  = $C011000E;
   cHIDPStatus_BUTTON_NOT_PRESSED       = $C011000F;
   cHIDPStatus_REPORT_DOES_NOT_EXIST    = $C0110010;
   cHIDPStatus_NOT_IMPLEMENTED          = $C0110020;

   cCollection_Physical    = #0;
   cCollection_Application = #1;
   cCollection_Logical     = #2;

   // Names for Bits in sets to indicate that the
   // following constants designate a bit in a set
   cBit0 = 0;  cBit8  =  8;  cBit16 = 16;  cBit24 = 24;
   cBit1 = 1;  cBit9  =  9;  cBit17 = 17;  cBit25 = 25;
   cBit2 = 2;  cBit10 = 10;  cBit18 = 18;  cBit26 = 26;
   cBit3 = 3;  cBit11 = 11;  cBit19 = 19;  cBit27 = 27;
   cBit4 = 4;  cBit12 = 12;  cBit20 = 20;  cBit28 = 28;
   cBit5 = 5;  cBit13 = 13;  cBit21 = 21;  cBit29 = 29;
   cBit6 = 6;  cBit14 = 14;  cBit22 = 22;  cBit30 = 30;
   cBit7 = 7;  cBit15 = 15;  cBit23 = 23;  cBit31 = 31;

   // i hope the bits are allocated from low to high
   bKMSLeftControl  = cBit0;
   bKMSLeftShift    = cBit1;
   bKMSLeftAlt      = cBit2;
   bKMSLeftGUI      = cBit3;
   bKMSRightControl = cBit4;
   bKMSRightShift   = cBit5;
   bKMSRightAlt     = cBit6;
   bKMSRightGUI     = cBit7;
   bKMSCapsLock     = cBit8;
   bKMSScollLock    = cBit9;
   bKMSNumLock      = cBit10;

   // same as above i hope this is the right bit
   bLCNIsAlias = cBit0;
   
type
   TVariantFlags = (Range, NoRange); // just for readable variant records

   // all HidP* functions return this type
   // the cHIDPStatus_* constants name the values
   THIDPResult = DWORD;

   // this creates a bitfield with a size of 4 bytes
   BitSet4 = set of 0..31;
   // the other possible bitfields (unused)
   BitSet2 = set of 0..15;
   BitSet1 = set of 0..7;

   // this is for HidD_GetConfiguration/HidD_SetConfiguration
   THIDDConfiguration = packed record
      cookie:         Pointer;
      size:           ULong;
      RingBufferSize: ULong;
   end;

   // HidD_GetAttributes fills this in
   THIDDAttributes = packed record
      cbSize:        ULong; // size of structure (set before call)
      VendorID:      Word;
      ProductID:     Word;
      VersionNumber: Word;
      // Additional fields will be added to the end of this structure.
   end;

   // PreparsedData is opaque
   PTHIDPPreparsedData = DWORD;

   // enums with 4 bytes for each value
{$Z4}
   THIDPReportType = (eHIDP_Input, eHIDP_Output, eHIDP_Feature);

   THIDPKeyboardDirection = (eHIDP_Keyboard_Break, eHIDP_Keyboard_Make);
{$Z1}

   // HidP_GetUsagesEx fills this in
   TUsageAndPage = packed record
      Usage:     TUsage;
      UsagePage: TUsage;
   end;

   PTUsageAndPage = ^TUsageAndPage;

   // HidP_GetSpecificButtonCaps fills this in
   THIDPButtonCaps = packed record
      UsagePage:         TUsage;
      ReportID:          Byte;
      IsAlias:           ByteBool;

      BitField:          Word;
      LinkCollection:    Word;   // A unique internal index pointer

      LinkUsage:         TUsage;
      LinkUsagePage:     TUsage;

      IsRange:           ByteBool;
      IsStringRange:     ByteBool;
      IsDesignatorRange: ByteBool;
      IsAbsolute:        ByteBool;

      Reserved:          array [0..9] of ULong;
   case TVariantFlags of
      Range:
         ( UsageMin,         UsageMax:      TUsage;
           StringMin,        StringMax,
           DesignatorMin,    DesignatorMax,
           DataIndexMin,     DataIndexMax:  Word );
      NoRange:
         ( Usage,            Reserved1:     TUsage;
           StringIndex,      Reserved2,
           DesignatorIndex,  Reserved3,
           DataIndex,        Reserved4:     Word );
   end;

   PTHIDPButtonCaps = ^THIDPButtonCaps;

   // HidP_GetSpecificValueCaps fills this in
   THIDPValueCaps = packed record
      UsagePage:         TUsage;
      ReportID:          Byte;
      IsAlias:           ByteBool;

      BitField:          Word;
      LinkCollection:    Word;   // A unique internal index pointer

      LinkUsage:         TUsage;
      LinkUsagePage:     TUsage;

      IsRange:           ByteBool;
      IsStringRange:     ByteBool;
      IsDesignatorRange: ByteBool;
      IsAbsolute:        ByteBool;

      HasNull:           ByteBool;        // Does this channel have a null report   union
      Reserved:          Byte;
      BitSize:           Word;            // How many bits are devoted to this value?

      ReportCount:       Word;            // See Note below.  Usually set to 1.
      Reserved0:         array [0..4] of Word;

      UnitsExp:          ULong;
      Units:             ULong;

      LogicalMin,   LogicalMax,
      PhysicalMin,  PhysicalMax: Integer;

   case TVariantFlags of
      Range:
         ( UsageMin,         UsageMax:      TUsage;
           StringMin,        StringMax,
           DesignatorMin,    DesignatorMax,
           DataIndexMin,     DataIndexMax:  Word );
      NoRange:
         ( Usage,            Reserved1:     TUsage;
           StringIndex,      Reserved2,
           DesignatorIndex,  Reserved3,
           DataIndex,        Reserved4:     Word );
   end;

   PTHIDPValueCaps = ^THIDPValueCaps;

   // HidP_GetLinkCollectionNodes fills this in
   // use bLCNIsAlias constant to check IsAlias
   THIDPLinkCollectionNode = packed record
      LinkUsage:        TUsage;
      LinkUsagePage:    TUsage;
      Parent:           Word;
      NumberOfChildren: Word;
      NextSibling:      Word;
      FirstChild:       Word;
      CollectionType:   Byte;    // As defined in 6.2.2.6 of HID spec
      IsAlias:          BitSet1; // This link node is an alias of the next link node.
      Reserved:         BitSet2;
      UserContext:      Pointer; // The user can hang his coat here.
   end;

   PTHIDPLinkCollectionNode = ^THIDPLinkCollectionNode;

   // Reports are simply handled as buffers of bytes
   PTHIDPReportDescriptor = PChar;

   // HidP_GetCaps fills this in
   THIDPCaps = packed record
      Usage:                     TUsage;
      UsagePage:                 TUsage;
      InputReportByteLength:     Word;
      OutputReportByteLength:    Word;
      FeatureReportByteLength:   Word;
      Reserved:                  array [0..16] of Word;

      NumberLinkCollectionNodes: Word;

      NumberInputButtonCaps:     Word;
      NumberInputValueCaps:      Word;
      NumberInputDataIndices:    Word;

      NumberOutputButtonCaps:    Word;
      NumberOutputValueCaps:     Word;
      NumberOutputDataIndices:   Word;

      NumberFeatureButtonCaps:   Word;
      NumberFeatureValueCaps:    Word;
      NumberFeatureDataIndices:  Word;
   end;

   // HidP_GetData and HidP_SetData use this
   THIDPData = packed record
      DataIndex: Word;
      Reserved:  Word;
   case Integer of
      0: (RawValue: ULong);
      1: (IsOn:     ByteBool);
   end;

   PTHIDPData = ^THIDPData;

   // use the bKMS* constants
   THIDPKeyboardModifierState = BitSet4;

   // callback function type for HidP_TranslateUsagesToI8042ScanCodes param
   THIDPInsertScanCodesFunc =
      function(
               Context:      Pointer; // Some caller supplied context
               NewScanCodes: PChar;   // A list of i8042 scan codes
               Length:       ULong    // the length of the scan code list
              ): Boolean; stdcall;

   procedure HidD_GetHidGuid
     (
      var HidGuid: TGUID
     ) stdcall; external hiddll name 'HidD_GetHidGuid';

   function HidD_GetPreparsedData
     (
          HidHandle:     THandle;
      var PreparsedData: PTHIDPPreparsedData
     ): LongBool; stdcall; external hiddll name 'HidD_GetPreparsedData';

   function HidD_FreePreparsedData
     (
          PreparsedData: PTHIDPPreparsedData
     ): LongBool; stdcall; external hiddll name 'HidD_FreePreparsedData';

   function HidD_GetConfiguration
     (
          HidHandle: THandle;
      var HidConfig: THIDDConfiguration;
          Size:      Integer
     ): LongBool; stdcall; external hiddll name 'HidD_GetConfiguration';

   function HidD_SetConfiguration
     (
          HidHandle: THandle;
      var HidConfig: THIDDConfiguration;
          Size:      Integer
     ): LongBool; stdcall; external hiddll name 'HidD_SetConfiguration';

   function HidD_FlushQueue
     (
          HidHandle: THandle
     ): LongBool; stdcall; external hiddll name 'HidD_FlushQueue';

   function HidD_GetFeature
     (
          HidHandle: THandle;
          Buffer:    PChar;
          Size:      Integer
     ): LongBool; stdcall; external hiddll name 'HidD_GetFeature';

   function HidD_SetFeature
     (
          HidHandle: THandle;
          Buffer:    PChar;
          Size:      Integer
     ): LongBool; stdcall; external hiddll name 'HidD_SetFeature';

   function HidD_GetNumInputBuffers
     (
          HidHandle: THandle;
      var NumBufs:   Integer
     ): LongBool; stdcall; external hiddll name 'HidD_GetNumInputBuffers';

   function HidD_SetNumInputBuffers
     (
          HidHandle: THandle;
          NumBufs:   Integer
     ): LongBool; stdcall; external hiddll name 'HidD_SetNumInputBuffers';

   function HidD_GetAttributes
     (
          HidHandle: THandle;
      var HidAttrs:  THIDDAttributes
     ): LongBool; stdcall; external hiddll name 'HidD_GetAttributes';

   function HidD_GetManufacturerString
     (
          HidHandle: THandle;
          buffer:    PWideChar;
          bufsize:   Integer
     ): LongBool; stdcall; external hiddll name 'HidD_GetManufacturerString';

   function HidD_GetProductString
     (
         HidHandle: THandle;
         buffer:    PWideChar;
         bufsize:   Integer
     ): LongBool; stdcall; external hiddll name 'HidD_GetProductString';

   function HidD_GetSerialNumberString
     (
          HidHandle: THandle;
          buffer:    PWideChar;
          bufsize:   Integer
     ): LongBool; stdcall; external hiddll name 'HidD_GetSerialNumberString';

   function HidD_GetPhysicalDescriptor
     (
          HidHandle: THandle;
          buffer:    PChar;
          bufsize:   Integer
     ): LongBool; stdcall; external hiddll name 'HidD_GetPhysicalDescriptor';

   function HidD_GetIndexedString
     (
          HidHandle: THandle;
          Index:     Integer;
          buffer:    PWideChar;
          bufsize:   Integer
     ): LongBool; stdcall; external hiddll name 'HidD_GetIndexedString';

   function HidP_GetCaps
     (
          PreparsedData: PTHIDPPreparsedData;
      var Capabilities:  THIDPCaps
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetCaps';

   function HidP_GetLinkCollectionNodes
     (
          LinkCollectionNodes:       PTHIDPLinkCollectionNode;
      var LinkCollectionNodesLength: ULong;
          PreparsedData:             PTHIDPPreparsedData
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetLinkCollectionNodes';

   function HidP_GetSpecificButtonCaps
     (
          ReportType:       THIDPReportType;
          UsagePage:        TUsage;      // Optional (0 => ignore)
          LinkCollection:   Word;        // Optional (0 => ignore)
          Usage:            TUsage;      // Optional (0 => ignore)
          ButtonCaps:       PTHIDPButtonCaps;
      var ButtonCapsLength: Word;
          PreparsedData:    PTHIDPPreparsedData
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetSpecificButtonCaps';

   function HidP_GetButtonCaps
     (
          ReportType:       THIDPReportType;
          ButtonCaps:       PTHIDPButtonCaps;
      var ButtonCapsLength: Word;
          PreparsedData:    PTHIDPPreparsedData
     ): THIDPResult; stdcall;

   function HidP_GetSpecificValueCaps
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage;      // Optional (0 => ignore)
          LinkCollection:  Word;        // Optional (0 => ignore)
          Usage:           TUsage;      // Optional (0 => ignore)
          ValueCaps:       PTHIDPValueCaps;
      var ValueCapsLength: Word;
          PreparsedData:   PTHIDPPreparsedData
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetSpecificValueCaps';

   function HidP_GetValueCaps
     (
          ReportType:      THIDPReportType;
          ValueCaps:       PTHIDPValueCaps;
      var ValueCapsLength: Word;
          PreparsedData:   PTHIDPPreparsedData
     ): THIDPResult; stdcall;

   function HidP_GetData
     (
          ReportType:      THIDPReportType;
          DataList:        PTHIDPData;
      var DataLength:      ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetData';

   function HidP_SetData
     (
          ReportType:      THIDPReportType;
          DataList:        PTHIDPData;
      var DataLength:      ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_SetData';

   function HidP_MaxDataListLength
     (
          ReportType:      THIDPReportType;
          PreparsedData:   PTHIDPPreparsedData
     ): ULong; stdcall; external hiddll name 'HidP_MaxDataListLength';

   function HidP_GetUsages
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage;
          LinkCollection:  Word; // Optional
          UsageList:       PTUsage;
      var UsageLength:     ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetUsages';

   function HidP_GetButtons
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage;
          LinkCollection:  Word; // Optional
          UsageList:       PTUsage;
      var UsageLength:     ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetUsages';

   function HidP_GetUsagesEx
     (
          ReportType:      THIDPReportType;
          LinkCollection:  Word; // Optional
          UsageList:       PTUsageAndPage;
      var UsageLength:     ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetUsagesEx';

   function HidP_GetButtonsEx
     (
          ReportType:      THIDPReportType;
          LinkCollection:  Word; // Optional
          UsageList:       PTUsageAndPage;
      var UsageLength:     ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetUsagesEx';

   function HidP_SetUsages
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage;
          LinkCollection:  Word; // Optional
          UsageList:       PTUsage;
      var UsageLength:     ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_SetUsages';

   function HidP_SetButtons
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage;
          LinkCollection:  Word; // Optional
          ButtonList:      PTUsage;
      var ButtonLength:    ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_SetUsages';

   function HidP_UnsetUsages
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage;
          LinkCollection:  Word; // Optional
          UsageList:       PTUsage;
      var UsageLength:     ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_UnsetUsages';

   function HidP_UnsetButtons
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage;
          LinkCollection:  Word; // Optional
          ButtonList:      PTUsage;
      var ButtonLength:    ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_UnsetUsages';

   function HidP_MaxUsageListLength
     (
          ReportType:    THIDPReportType;
          UsagePage:     TUsage; // Optional
          PreparsedData: PTHIDPPreparsedData
     ): ULong; stdcall; external hiddll name 'HidP_MaxUsageListLength';

   function HidP_MaxButtonListLength
     (
          ReportType:    THIDPReportType;
          UsagePage:     TUsage; // Optional
          PreparsedData: PTHIDPPreparsedData
     ): ULong; stdcall; external hiddll name 'HidP_MaxUsageListLength';

   function HidP_GetUsageValue
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage; // Optional
          LinkCollection:  Word; // Optional
          Usage:           TUsage;
      var UsageValue:      ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetUsageValue';

   function HidP_GetScaledUsageValue
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage; // Optional
          LinkCollection:  Word; // Optional
          Usage:           TUsage;
      var UsageValue:      Integer;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_GetScaledUsageValue';

   function HidP_GetUsageValueArray
     (
          ReportType:           THIDPReportType;
          UsagePage:            TUsage; // Optional
          LinkCollection:       Word; // Optional
          Usage:                TUsage;
          UsageValue:           PChar;
          UsageValueByteLength: Word;
          PreparsedData:        PTHIDPPreparsedData;
          Report:               PChar;
          ReportLength:         ULong
      ): THIDPResult; stdcall; external hiddll name 'HidP_GetUsageValueArray';

   function HidP_SetUsageValue
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage; // Optional
          LinkCollection:  Word; // Optional
          Usage:           TUsage;
          UsageValue:      ULong;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_SetUsageValue';

   function HidP_SetScaledUsageValue
     (
          ReportType:      THIDPReportType;
          UsagePage:       TUsage; // Optional
          LinkCollection:  Word; // Optional
          Usage:           TUsage;
          UsageValue:      Integer;
          PreparsedData:   PTHIDPPreparsedData;
          Report:          PChar;
          ReportLength:    ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_SetScaledUsageValue';

   function HidP_SetUsageValueArray
     (
          ReportType:           THIDPReportType;
          UsagePage:            TUsage; // Optional
          LinkCollection:       Word; // Optional
          Usage:                TUsage;
          UsageValue:           PChar;
          UsageValueByteLength: Word;
          PreparsedData:        PTHIDPPreparsedData;
          Report:               PChar;
          ReportLength:         ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_SetUsageValueArray';

   function HidP_UsageListDifference
     (
          PreviousUsageList: PTUsage;
          CurrentUsageList:  PTUsage;
          BreakUsageList:    PTUsage;
          MakeUsageList:     PTUsage;
          UsageListLength:   ULong
     ): THIDPResult; stdcall; external hiddll name 'HidP_UsageListDifference';

   function HidP_TranslateUsagesToI8042ScanCodes
     (
          ChangedUsageList:     PTUsage;
          UsageListLength:      ULong;
          KeyAction:            THIDPKeyboardDirection;
      var ModifierState:        THIDPKeyboardModifierState;
          InsertCodesProcedure: THIDPInsertScanCodesFunc;
          InsertCodesContext:   Pointer
     ): THIDPResult; stdcall; external hiddll name 'HidP_TranslateUsagesToI8042ScanCodes';

implementation

   function HidP_GetButtonCaps
     (
          ReportType:       THIDPReportType;
          ButtonCaps:       PTHIDPButtonCaps;
      var ButtonCapsLength: Word;
          PreparsedData:    PTHIDPPreparsedData
     ): THIDPResult; stdcall;
   begin
      HidP_GetButtonCaps := HidP_GetSpecificButtonCaps(ReportType,0,0,0,ButtonCaps,ButtonCapsLength,PreparsedData);
   end;

   function HidP_GetValueCaps
     (
          ReportType:      THIDPReportType;
          ValueCaps:       PTHIDPValueCaps;
      var ValueCapsLength: Word;
          PreparsedData:   PTHIDPPreparsedData
     ): THIDPResult; stdcall;
   begin
      HidP_GetValueCaps := HidP_GetSpecificValueCaps(ReportType,0,0,0,ValueCaps,ValueCapsLength,PreparsedData);
   end;

end.
