{
* a Class interface for HID access
* a controller class THidDeviceController manages all THidDevice objects
* create a controller instance (later as component) which creates
* THidDevice objects for all found HID devices
* on plug or unplug of devices the controller updates its list
* the THidDevice object of an unplugged device ist not freed but
* its methods cease to work through a flag
}

unit HidClass;

interface

uses
   Windows, Messages, Classes,
   Dialogs, Forms, Registry, SysUtils,
   DevChange, SetupAPI, Hid, HidUsage;

const
   // strings from the registry for CheckOutByClass 
   cKeyboardClass = 'Keyboard';
   cMouseClass    = 'Mouse';

type
   THidDeviceController = class;

   // the representation of a HID device

   THidDevice = class(TObject)
    private
      // internal control variables 
      FMyController       : THidDeviceController;
      FIsPluggedIn        : Boolean;
      FIsCheckedOut       : Boolean;
      FIsEnumerated       : Boolean;
      FHidHandle          : THandle;
      // internal properties part 
      FDevRegKey          : DWORD;
      FDevicePath         : String;
      FHardWareKey        : String;
      FRegDescr           : String;
      FRegClass           : String;
      FVendorName         : String;
      FProductName        : String;
      FSerialNumber       : String;
      FDeviceStrings      : TStringList;
      FPreparsedData      : PTHIDPPreparsedData;
      FAttributes         : THIDDAttributes;
      FCaps               : THIDPCaps;
      FConfiguration      : THIDDConfiguration;
      FNumInputBuffers    : Integer;
      FLinkCollection     : PTHIDPLinkCollectionNode;
      FLinkCollectionNodes: TList;
      FMaxDataListLength  : ULong;
      FMaxUsageListLength : ULong;
      FMaxButtonListLength: ULong;
      FReportTypeParam    : THIDPReportType;
      FUsagePageParam     : TUsage;
      FLinkCollectionParam: Word;
      FUsageParam         : TUsage;
      FUnplug             : TNotifyEvent;

      // file open is internal
      function  OpenFile: Boolean;
      // internal property implementors
      procedure SetConfiguration  (Config: THIDDConfiguration);
      procedure SetNumInputBuffers(const Num: Integer);
      procedure SetReportTypeParam(const ReportType: THIDPReportType);
      procedure SetUsagePageParam (const UsagePage: TUsage);
      // constructor is hidden!
      constructor Create   (DevicePath: String; DevID: DWORD);

    protected
      // internal property implementors
      procedure   DoUnplug;
      procedure   SetUnplug(const value: TNotifyEvent);

    public

    published
      destructor Destroy; override;

      // methods
      procedure CloseFile;
      function  FlushQueue:                                                   Boolean;
      function  GetFeature           (Buffer: PChar; const Size: Integer):    Boolean;
      function  SetFeature           (Buffer: PChar; const Size: Integer):    Boolean;
      function  GetPhysicalDescriptor(Buffer: PChar; const Size: Integer):    Boolean;
      function  GetSpecificButtonCaps(ButtonCaps: PTHIDPButtonCaps; var Count: Word): THIDPResult;
      function  GetButtonCaps        (ButtonCaps: PTHIDPButtonCaps; var Count: Word): THIDPResult;
      function  GetSpecificValueCaps (ValueCaps:  PTHIDPValueCaps;  var Count: Word): THIDPResult;
      function  GetValueCaps         (ValueCaps:  PTHIDPValueCaps;  var Count: Word): THIDPResult;
      function  GetData              (DataList: PTHIDPData; var DataLength: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  SetData              (DataList: PTHIDPData; var DataLength: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  GetUsages            (UsageList: PTUsage; var UsageLength: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  GetButtons           (UsageList: PTUsage; var UsageLength: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  GetUsagesEx          (UsageList: PTUsageAndPage; var UsageLength: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  GetButtonsEx         (UsageList: PTUsageAndPage; var UsageLength: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  SetUsages            (UsageList: PTUsage; var UsageLength: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  SetButtons           (UsageList: PTUsage; var UsageLength: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  UnsetUsages          (UsageList: PTUsage; var UsageLength: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  UnsetButtons         (UsageList: PTUsage; var UsageLength: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  GetUsageValue        (var UsageValue: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  GetScaledUsageValue  (var UsageValue: Integer;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  GetUsageValueArray   (UsageValue: PChar; UsageValueByteLength: Word;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  SetUsageValue        (UsageValue: ULong;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  SetScaledUsageValue  (UsageValue: Integer;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
      function  SetUsageValueArray   (UsageValue: PChar; UsageValueByteLength: Word;
                                      Report: PChar; ReportLength: ULong)           : THIDPResult;
                                      
      // management properties
      property IsPluggedIn        : Boolean             read FIsPluggedIn;
      property IsCheckedOut       : Boolean             read FIsCheckedOut;
      // all the precollected info from the device as properties 
      property DevRegKey          : DWORD               read FDevRegKey;
      property DevicePath         : String              read FDevicePath;
      property HardWareKey        : String              read FHardWareKey;
      property RegDescr           : String              read FRegDescr;
      property RegClass           : String              read FRegClass;
      property VendorName         : String              read FVendorName;
      property ProductName        : String              read FProductName;
      property SerialNumber       : String              read FSerialNumber;
      property DeviceStrings      : TStringList         read FDeviceStrings;
      property PreparsedData      : PTHIDPPreparsedData read FPreparsedData;
      property Attributes         : THIDDAttributes     read FAttributes;
      property Caps               : THIDPCaps           read FCaps;
      property LinkCollectionNodes: TList               read FLinkCollectionNodes;
      // connected to ReportTypeParam and UsagePageParam
      property MaxDataListLength  : ULong               read FMaxDataListLength;
      property MaxUsageListLength : ULong               read FMaxUsageListLength;
      property MaxButtonListLength: ULong               read FMaxButtonListLength;
      // changeable info as properties
      property Configuration      : THIDDConfiguration  read FConfiguration       write SetConfiguration;
      property NumInputBuffers    : Integer             read FNumInputBuffers     write SetNumInputBuffers;
      // parameter reduction properties for the methods
      property ReportTypeParam    : THIDPReportType     read FReportTypeParam     write SetReportTypeParam;
      property UsagePageParam     : TUsage              read FUsagePageParam      write SetUsagePageParam;
      property LinkCollectionParam: Word                read FLinkCollectionParam write FLinkCollectionParam;
      property UsageParam         : TUsage              read FUsageParam          write FUsageParam;
      // the only event property
      property OnUnplug           : TNotifyEvent        read FUnplug              write SetUnplug;
   end;

   THidDeviceEvent = function(HidDev: THidDevice; Index: Integer): Boolean of object;

   // controller class to manage all HID devices 

   THidDeviceController = class(TComponent)
    private
      // internal property part 
      FHidGuid          : TGUID;
      FDeviceChangeEvent: TNotifyEvent;
      FEnumerateEvent   : THidDeviceEvent;
      // internal list of all HID device objects 
      FList             : TList;
      // internal worker functions 
      function    CheckThisOut(var HidDev: THidDevice; Index: Integer; Check: Boolean): Boolean;
      procedure   FillInList  (var List: TList);
      function    EventPipe   (var Msg: TMessage): Boolean;

    protected
      // internal property implementors 
      procedure   DoDeviceChange;
      function    DoEnumerate         (HidDev: THidDevice; Index: Integer): Boolean;
      procedure   SetDeviceChangeEvent(const value: TNotifyEvent);
      procedure   SetEnumerate        (const value: THidDeviceEvent);

    published
      // normal constructor/destructor 
      constructor Create(AOwner: TComponent); override;
      destructor  Destroy;                    override;

      // methods to hand out HID device objects 
      function    CheckOutByProductName(var HidDev: THidDevice; ProductName: String) : Boolean;
      function    CheckOutByVendorName (var HidDev: THidDevice; VendorName:  String) : Boolean;
      function    CheckOutByClass      (var HidDev: THidDevice; ClassName:   String) : Boolean;
      function    CheckOutByID         (var HidDev: THidDevice; Vid, Pid:    Integer): Boolean;
      function    CheckOutByIndex      (var HidDev: THidDevice; const Index: Integer): Boolean;
      function    CheckOut             (var HidDev: THidDevice)                      : Boolean;
      procedure   CheckIn              (var HidDev: THidDevice);
      // iterate over the HID devices 
      function    Enumerate: Integer;

      // just to be complete the GUID 
      property    HidGuid:        TGUID           read FHidGuid;
      // the central event for HID device changes
      property    OnDeviceChange: TNotifyEvent    read FDeviceChangeEvent write SetDeviceChangeEvent;
      // the iterator event 
      property    OnEnumerate:    THidDeviceEvent read FEnumerateEvent    write SetEnumerate;
   end;

implementation

//== THidDevice =======================================================

// get the registry path to the device info 

function DevIDToHardWareKey(DevID: DWORD): String;
var
   reg: TRegistry;
begin
   reg := TRegistry.Create;
   reg.RootKey := HKEY_DYN_DATA;
   reg.OpenKey('Config Manager\Enum\' + Format('%.8x',[DevID]),False);
   DevIDToHardWareKey := reg.ReadString('HardWareKey');
   reg.CloseKey;
end;

// get some device info from the registry 

function HardWareKeyToString(key: String; val: String): String;
var
   reg: TRegistry;
begin
   reg := TRegistry.Create;
   reg.RootKey := HKEY_LOCAL_MACHINE;
   reg.OpenKey('Enum\' + key,False);
   HardWareKeyToString := reg.ReadString(val);
   reg.CloseKey;
end;

//-- THidDevice: basics and internals ---------------------------------

{
* create and fill in a HidDevice object
* the constructor is only accessible from THidController
* DevicePath names the device "file"
* DevID is a handle (registry key from dynamic registry data) to the
* individual device (differs even for two devices of the same kind)
}

constructor THidDevice.Create(DevicePath: String; DevID: DWORD);
var
   i     : Integer;
   len   : ULong;
   ptr   : PTHIDPLinkCollectionNode;
   buffer: array [0..255] of WideChar;
begin
   inherited Create;

   // initialize private data 
   FIsPluggedIn         := True;
   FIsCheckedOut        := False;
   FIsEnumerated        := False;
   FHidHandle           := THandle(-1);
   FDevicePath          := DevicePath;
   FReportTypeParam     := eHIDP_Input;
   FUsagePageParam      := 0;
   FLinkCollectionParam := 0;
   FUsageParam          := 0;
   FUnplug              := Nil;
   FHidHandle := CreateFile(PChar(FDevicePath), GENERIC_READ OR GENERIC_WRITE,
                            FILE_SHARE_READ OR FILE_SHARE_WRITE, Nil,OPEN_EXISTING,0,0);
   if FHidHandle <> THandle(-1) then
   begin
      // get all device data through HID functions
      // this eliminates the need to redeclare them as methods
      FDevRegKey   := DevID;
      FHardWareKey := DevIDToHardWareKey(DevID);
      FRegDescr    := HardWareKeyToString(FHardWareKey,'DeviceDesc');
      FRegClass    := HardWareKeyToString(FHardWareKey,'Class');
      FAttributes.cbSize := SizeOf(THIDDAttributes);
      HidD_GetPreparsedData  (FHidHandle,FPreparsedData);
      HidD_GetAttributes     (FHidHandle,FAttributes);
      HidD_GetConfiguration  (FHidHandle,FConfiguration,SizeOf(THIDDConfiguration));
      HidD_GetNumInputBuffers(FHidHandle,FNumInputBuffers);
      HidP_GetCaps(FPreparsedData,FCaps);
      if HidD_GetManufacturerString(FHidHandle,buffer,256) then
         FVendorName := WideCharToString(buffer);
      if HidD_GetProductString(FHidHandle,buffer,256) then
         FProductName := WideCharToString(buffer);
      if HidD_GetSerialNumberString(FHidHandle,buffer,256) then
         FSerialNumber := WideCharToString(buffer);
      // put all device strings in a TStringList 
      FDeviceStrings := TStringList.Create;
      i := 1;
      while HidD_GetIndexedString(FHidHandle,i,buffer,256) do
      begin
         FDeviceStrings.Add(WideCharToString(buffer));
         Inc(i);
      end;
      // put all LinkCollectionNodes in a TList
      FLinkCollectionNodes := TList.Create;
      len := FCaps.NumberLinkCollectionNodes;
      GetMem(FLinkCollection,len*SizeOf(THIDPLinkCollectionNode));
      HidP_GetLinkCollectionNodes(FLinkCollection,len,FPreparsedData);
      ptr := FLinkCollection;
      for i := 1 to len do
      begin
         FLinkCollectionNodes.Add(ptr);
         ptr := PTHIDPLinkCollectionNode((PChar(ptr)+SizeOf(THIDPLinkCollectionNode)));
      end;
   end;
   // the file is closed to stop using up resources
   CloseFile;
end;

{
* if a THidDevice is destroyed the THidController has to be informed
* if the device is plugged in this THidDevice instance is destroyed
* but another instance is created in the controller list
}

destructor THidDevice.Destroy;
var
   i: Integer;
begin
   // free the data which needs special handling 
   CloseFile;
   HidD_FreePreparsedData(FPreparsedData);
   FLinkCollectionNodes.Free;
   FreeMem(FLinkCollection);
   // if controller exists 
   if FMyController <> Nil then
      with FMyController do
      begin
         // delete device from controller list 
         for i := 0 to FList.Count-1 do
            if FList.Items[i] = Self then
            begin
               // if device is plugged in create a checked in copy 
               if IsPluggedIn then
               begin
                  FList.Items[i] := THidDevice.Create(DevicePath,FDevRegKey);
                  THidDevice(FList.Items[i]).FMyController := FMyController;
               end
               else
                  FList.Delete(i);
               Break;
            end;
      end;

   inherited Destroy;
end;

// open the device "file" (for the other methods) 

function THidDevice.OpenFile: Boolean;
begin
   // check if open allowed (propagates this state)
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      if FHidHandle = THandle(-1) then // if not already opened
         FHidHandle := CreateFile(PChar(FDevicePath), GENERIC_READ OR GENERIC_WRITE,
                                  FILE_SHARE_READ OR FILE_SHARE_WRITE, Nil,OPEN_EXISTING,0,0);
   Result := FHidHandle <> THandle(-1);
end;

// implement OnUnplug event

procedure THidDevice.DoUnplug;
begin
   CloseFile;
   FIsPluggedIn := False;
   if Assigned(FUnplug) AND FIsCheckedOut then
      FUnplug(Self);
end;

// assign the OnUnplug event 

procedure THidDevice.SetUnplug(const value: TNotifyEvent);
begin
   if @value <> @FUnplug then
      FUnplug := value;
end;

// implement property NumInputBuffers write 

procedure THidDevice.SetNumInputBuffers(const Num: Integer);
begin
   if OpenFile then
      if HidD_SetNumInputBuffers(FHidHandle,Num) then
         FNumInputBuffers := Num;
end;

// implement property ReportTypeParam write

procedure THidDevice.SetReportTypeParam(const ReportType: THIDPReportType);
begin
   FReportTypeParam := ReportType;
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
   begin
      FMaxDataListLength   := HidP_MaxDataListLength  (ReportTypeParam,PreparsedData);
      FMaxUsageListLength  := HidP_MaxUsageListLength (ReportTypeParam,UsagePageParam,PreparsedData);
      FMaxButtonListLength := HidP_MaxButtonListLength(ReportTypeParam,UsagePageParam,PreparsedData);
   end;
end;

// implement property UsagePageParam write

procedure THidDevice.SetUsagePageParam (const UsagePage: TUsage);
begin
   FUsagePageParam := UsagePage;
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
   begin
      FMaxDataListLength   := HidP_MaxDataListLength  (ReportTypeParam,PreparsedData);
      FMaxUsageListLength  := HidP_MaxUsageListLength (ReportTypeParam,UsagePageParam,PreparsedData);
      FMaxButtonListLength := HidP_MaxButtonListLength(ReportTypeParam,UsagePageParam,PreparsedData);
   end;
end;

// implement property Configuration write

procedure THidDevice.SetConfiguration(Config: THIDDConfiguration);
begin
   if OpenFile then
      if HidD_SetConfiguration(FHidHandle,Config,Sizeof(THIDDConfiguration)) then
         FConfiguration := Config;
end;

//-- THidDevice methods -----------------------------------------------

{
* generally the parameter count of the methods is reduced with the Param propertiies
* first assign the Param properties the desired value
* then call a method
* normally you will address the same Usage, UsagePage, ReportType or LinkCollection
* with more than one method
*
* the methods will open the device file when needed
* this file is not closed until unplug or destruction to speed up access
}

// close the device "file"
// if you want to open the file directly close this to get access

procedure THidDevice.CloseFile;
begin
   if FHidHandle <> THandle(-1) then
      CloseHandle(FHidHandle);
   FHidHandle := THandle(-1);
end;

function THidDevice.FlushQueue: Boolean;
begin
   Result := False;
   if OpenFile then
      Result := HidD_FlushQueue(FHidHandle);
end;

function THidDevice.GetFeature(Buffer: PChar; const Size: Integer): Boolean;
begin
   Result := False;
   if OpenFile then
      Result := HidD_GetFeature(FHidHandle,Buffer,Size);
end;

function THidDevice.SetFeature(Buffer: PChar; const Size: Integer): Boolean;
begin
   Result := False;
   if OpenFile then
      Result := HidD_SetFeature(FHidHandle,Buffer,Size);
end;

function THidDevice.GetPhysicalDescriptor(Buffer: PChar; const Size: Integer): Boolean;
begin
   Result := False;
   if OpenFile then
      Result := HidD_GetPhysicalDescriptor(FHidHandle,Buffer,Size);
end;

function THidDevice.GetSpecificButtonCaps(ButtonCaps: PTHIDPButtonCaps; var Count: Word): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetSpecificButtonCaps(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageParam,ButtonCaps,Count,PreparsedData);
end;

function THidDevice.GetButtonCaps(ButtonCaps: PTHIDPButtonCaps; var Count: Word): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetButtonCaps(ReportTypeParam,ButtonCaps,Count,PreparsedData);
end;

function THidDevice.GetSpecificValueCaps(ValueCaps: PTHIDPValueCaps; var Count: Word): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetSpecificValueCaps(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageParam,ValueCaps,Count,PreparsedData);
end;

function THidDevice.GetValueCaps(ValueCaps: PTHIDPValueCaps; var Count: Word): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetValueCaps(ReportTypeParam,ValueCaps,Count,PreparsedData);
end;

function THidDevice.GetData(DataList: PTHIDPData; var DataLength: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetData(ReportTypeParam,DataList,DataLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.SetData(DataList: PTHIDPData; var DataLength: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_SetData(ReportTypeParam,DataList,DataLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.GetUsages(UsageList: PTUsage; var UsageLength: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetUsages(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageList,UsageLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.GetButtons(UsageList: PTUsage; var UsageLength: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetButtons(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageList,UsageLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.GetUsagesEx(UsageList: PTUsageAndPage; var UsageLength: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetUsagesEx(ReportTypeParam,LinkCollectionParam,UsageList,UsageLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.GetButtonsEx(UsageList: PTUsageAndPage; var UsageLength: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetButtonsEx(ReportTypeParam,LinkCollectionParam,UsageList,UsageLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.SetUsages(UsageList: PTUsage; var UsageLength: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_SetUsages(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageList,UsageLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.SetButtons(UsageList: PTUsage; var UsageLength: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_SetButtons(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageList,UsageLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.UnsetUsages(UsageList: PTUsage; var UsageLength: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_UnsetUsages(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageList,UsageLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.UnsetButtons(UsageList: PTUsage; var UsageLength: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_UnsetButtons(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageList,UsageLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.GetUsageValue(var UsageValue: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetUsageValue(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageParam,
                                   UsageValue,PreparsedData,Report,ReportLength);
end;

function THidDevice.GetScaledUsageValue(var UsageValue: Integer;Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetScaledUsageValue(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageParam,
                                         UsageValue,PreparsedData,Report,ReportLength);
end;

function THidDevice.GetUsageValueArray(UsageValue: PChar; UsageValueByteLength: Word; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_GetUsageValueArray(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageParam,
                                        UsageValue,UsageValueByteLength,PreparsedData,Report,ReportLength);
end;

function THidDevice.SetUsageValue(UsageValue: ULong; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_SetUsageValue(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageParam,
                                   UsageValue,PreparsedData,Report,ReportLength);
end;

function THidDevice.SetScaledUsageValue(UsageValue: Integer;Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_SetScaledUsageValue(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageParam,
                                         UsageValue,PreparsedData,Report,ReportLength);
end;

function THidDevice.SetUsageValueArray(UsageValue: PChar; UsageValueByteLength: Word; Report: PChar; ReportLength: ULong): THIDPResult;
begin
   Result := cHIDPStatus_NULL;  // for not plugged in
   if IsPluggedIn AND (IsCheckedOut OR FIsEnumerated) then
      Result := HidP_SetUsageValueArray(ReportTypeParam,UsagePageParam,LinkCollectionParam,UsageParam,
                                        UsageValue,UsageValueByteLength,PreparsedData,Report,ReportLength);
end;

//== THidDeviceController =============================================

// internal worker function to find all Hid devices and create their objects

procedure THidDeviceController.FillInList(var List: TList);
var
   PnPHandle              : THandle;
   DevData                : TSPDevInfoData;
   DeviceInterfaceData    : TSPDeviceInterfaceData;
   FunctionClassDeviceData: TSPDeviceInterfaceDetailData;
   Success                : Integer;
   devn                   : Integer;
   BytesReturned          : DWORD;
   HidDev                 : THidDevice;
begin
   // create list 
   List := TList.Create;

   // First, get class identifier 
   HidD_GetHidGuid(FHidGuid);

   // Get a handle for the Plug and Play node and request currently active HID devices 
   PnPHandle := SetupDiGetClassDevs(FHidGuid,Nil,0,cDIGCFPresent OR cDIGCFDeviceInterface);
   if PnPHandle = THandle(-1) then
      exit;
   devn := 0;
   repeat
      DeviceInterfaceData.cbSize := SizeOf(TSPDeviceInterfaceData);
      // Is there a HID device at this table entry? 
      Success := SetupDiEnumDeviceInterfaces(PnPHandle,Nil,FHidGuid,devn,DeviceInterfaceData);
      if Success = 1 then
      begin
         FunctionClassDeviceData.cbSize := 5;
         DevData.cbSize := SizeOf(DevData);
         if SetupDiGetDeviceInterfaceDetail(PnPHandle,DeviceInterfaceData,FunctionClassDeviceData,256,BytesReturned,@DevData) = 1 then
         begin
            // create HID device object and add it to the device list 
            HidDev := THidDevice.Create(FunctionClassDeviceData.DevicePath,DevData.DevInst);
            HidDev.FMyController := Self;
            List.Add(HidDev);
            Inc(devn);
         end;
      end;
   until Success <> 1;
   CloseHandle(PnPHandle);
end;

// the controller fills its list on creation and connects the event pipe 

constructor THidDeviceController.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FDeviceChangeEvent := Nil;
   FEnumerateEvent    := Nil;
   FillInList(FList);
   Application.HookMainWindow(EventPipe);
end;

// unplug or kill all controlled THidDevices on controller destruction 

destructor THidDeviceController.Destroy;
var
   i     : Integer;
   HidDev: THidDevice;
begin
   // unhook event pipe
   Application.UnhookMainWindow(EventPipe);
   for i := 0 to FList.Count-1 do
   begin
      HidDev := FList.Items[i];
      with HidDev do
      begin
         // set to uncontrolled 
         FMyController := Nil;
         if IsCheckedOut then
            // pull the plug for checked out THidDevices 
            DoUnplug
         else
            // kill THidDevices which are not checked out 
            Free;
      end;
   end;
   FList.Free;

   inherited Destroy;
end;

// gets all the windows events/messages directly 

function THidDeviceController.EventPipe(var Msg: TMessage): Boolean;
begin
   Result := False;
   // sort out WM_DEVICECHANGE : DBT_DEVNODES_CHANGED 
   if (Msg.Msg = WM_DEVICECHANGE) AND (TWMDeviceChange(Msg).Event = DBT_DEVNODES_CHANGED) then
      DoDeviceChange;
end;

// implements OnDeviceChange event 

procedure THidDeviceController.DoDeviceChange;
var
   i      : Integer;
   j      : Integer;
   NewList: TList;
   HidDev : THidDevice;
   Changed: Boolean;
begin
   Changed := False;
   // get new device list
   FillInList(NewList);
   // unplug devices in FList which are not in NewList
   for i := 0 to FList.Count-1 do
   begin
      HidDev := FList.Items[i];
      for j := 0 to NewList.Count-1 do
         if (THidDevice(NewList.Items[j]).FDevRegKey = HidDev.FDevRegKey) AND HidDev.IsPluggedIn then
         begin
            HidDev := Nil;
            Break;
         end;
      if HidDev <> Nil then
      begin
         HidDev.DoUnplug;
         Changed := True;
      end;
   end;
   // delete devices from NewList which are in FList
   for i := 0 to NewList.Count-1 do
      for j := 0 to FList.Count-1 do
         if (THidDevice(NewList[i]).FDevRegKey = THidDevice(FList[j]).FDevRegKey) AND THidDevice(FList[j]).IsPluggedIn then
         begin
            THidDevice(NewList[i]).FMyController := Nil; // prevent Free/Destroy from accessing this controller
            THidDevice(NewList[i]).Free;
            NewList[i] := Nil;
            Break;
         end;
   // add the remains in NewList to FList
   for i := 0 to NewList.Count-1 do
      if NewList[i] <> Nil then
      begin
         FList.Add(NewList[i]);
         Changed := True;
      end;
   NewList.Free;  // throw away helper list

   if Assigned(FDeviceChangeEvent) AND Changed then
      FDeviceChangeEvent(Self);
end;

// assign OnDeviceChange and immediately send one

procedure THidDeviceController.SetDeviceChangeEvent(const value: TNotifyEvent);
begin
   if @value <> @FDeviceChangeEvent then
   begin
      FDeviceChangeEvent := value;
      if Assigned(FDeviceChangeEvent) then
         FDeviceChangeEvent(Self);
   end;
end;

// implement OnEnumerate event 

function THidDeviceController.DoEnumerate(HidDev: THidDevice; Index: Integer): Boolean;
begin
   Result := False;
   if Assigned(FEnumerateEvent) then
   begin
      HidDev.FIsEnumerated := True;
      Result := FEnumerateEvent(HidDev,Index);
      HidDev.FIsEnumerated := False;
      if NOT HidDev.FIsCheckedOut then
         HidDev.CloseFile;
   end;
end;

// assign OnEnumerate event 

procedure THidDeviceController.SetEnumerate(const value: THidDeviceEvent);
begin
   if @value <> @FEnumerateEvent then
      FEnumerateEvent := value;
end;

// send an OnEnumerate event for ALL controlled HidDevices       
// it is explicitly allowed to check out any device in the event 

function THidDeviceController.Enumerate: Integer;
var
   i: Integer;
begin
   Result := 0;
   for i := 0 to FList.Count-1 do
      if THidDevice(FList[i]).IsPluggedIn then
      begin
         Inc(Result);
         if NOT DoEnumerate(FList[i],i) then
            Break;
      end;
end;

//-- THidDeviceController methods -------------------------------------

// internal worker function to check out a THidDevice

function THidDeviceController.CheckThisOut(var HidDev: THidDevice; Index: Integer; Check: Boolean): Boolean;
begin
   Result := Check AND NOT THidDevice(FList.Items[Index]).IsCheckedOut;
   if Result then
   begin
      HidDev := FList[Index];
      HidDev.FIsCheckedOut := True;
   end;
end;

// method CheckOutByProductName hands out the first HidDevice with the right ProductName

function THidDeviceController.CheckOutByProductName(var HidDev: THidDevice; ProductName: String): Boolean;
var
   i: Integer;
begin
   Result := False;
   for i := 0 to FList.Count-1 do
   begin
      Result := CheckThisOut(HidDev,i,(ProductName = THidDevice(FList[i]).ProductName) AND (ProductName <> ''));
      if Result then
         Break;
   end;
end;

// method CheckOutByVendorName hands out the first HidDevice with the right VendorName 

function THidDeviceController.CheckOutByVendorName(var HidDev: THidDevice; VendorName: String): Boolean;
var
   i: Integer;
begin
   Result := False;
   for i := 0 to FList.Count-1 do
   begin
      Result := CheckThisOut(HidDev,i,(VendorName = THidDevice(FList[i]).VendorName) AND (VendorName <> ''));
      if Result then
         Break;
   end;
end;

// method CheckOutByClass hands out the first HidDevice with the right Class 
// Class comes from the registry (examples: 'Mouse', 'Keyboard')             

function THidDeviceController.CheckOutByClass(var HidDev: THidDevice; ClassName: String): Boolean;
var
   i: Integer;
begin
   Result := False;
   for i := 0 to FList.Count-1 do
   begin
      Result := CheckThisOut(HidDev,i,(ClassName = THidDevice(FList[i]).RegClass) AND (ClassName <> ''));
      if Result then
         Break;
   end;
end;

// method CheckOutByID hands out the first HidDevice with the right VendorID and ProductID 
// Pid = -1 matches all ProductIDs                                                          

function THidDeviceController.CheckOutByID(var HidDev: THidDevice; Vid, Pid: Integer): Boolean;
var
   i: Integer;
begin
   Result := False;
   for i := 0 to FList.Count-1 do
   begin
      Result := CheckThisOut(HidDev,i,(Vid = THidDevice(FList[i]).Attributes.VendorID) AND ((Pid = THidDevice(FList[i]).Attributes.ProductID) OR (Pid = -1)));
      if Result then
         Break;
   end;
end;

// method CheckOutByIndex hands out the HidDevice in the list with the right index 
// this is mainly for check out during OnEnumerate                                 

function THidDeviceController.CheckOutByIndex(var HidDev: THidDevice; const Index: Integer): Boolean;
begin
   Result := False;
   if (Index >= 0) AND (Index < FList.Count) then
      Result := CheckThisOut(HidDev,Index,True);
end;

// method CheckOut simply hands out the first HidDevice in the list 

function THidDeviceController.CheckOut(var HidDev: THidDevice): Boolean;
var
   i: Integer;
begin
   Result := False;
   for i := 0 to FList.Count-1 do
   begin
      Result := CheckThisOut(HidDev,i,True);
      if Result then
         Break;
   end;
end;

// method CheckIn hands a checked out HidDevice back in

procedure THidDeviceController.CheckIn(var HidDev: THidDevice);
begin
   HidDev.CloseFile;
   if HidDev.IsPluggedIn then
      HidDev.FIsCheckedOut := False
   else
      HidDev.Free;
   HidDev := Nil;
end;

end.
