// HidUsage.pas contains all the consts from HidUsage.h
// the TUsage type is declared.
// some sets of constants have been completed (mainly keys)
// the constants are for Usage, UsagePage and Usage values
// that is they describe the real data you will get from USB devices
//
// for real information read "USB HID Usage Tables 1.1rc3.pdf"
// or newer versions thereof

unit HidUsage;

interface

uses Windows; // for Word = TUsage

//
// Usage Pages
//

const
   cHIDUsagePage_GENERIC      = $01;
   cHIDUsagePage_SIMULATION   = $02;
   cHIDUsagePage_VR           = $03;
   cHIDUsagePage_SPORT        = $04;
   cHIDUsagePage_GAME         = $05;
   cHIDUsagePage_KEYBOARD     = $07;
   cHIDUsagePage_LED          = $08;
   cHIDUsagePage_BUTTON       = $09;
   cHIDUsagePage_ORDINAL      = $0A;
   cHIDUsagePage_TELEPHONY    = $0B;
   cHIDUsagePage_CONSUMER     = $0C;
   cHIDUsagePage_DIGITIZER    = $0D;
   cHIDUsagePage_UNICODE      = $10;
   cHIDUsagePage_ALPHANUMERIC = $14;


//
// Usages from Generic Desktop Page (0x01)
//

   cHIDUsageGeneric_POINTER    = $01;
   cHIDUsageGeneric_MOUSE      = $02;
   cHIDUsageGeneric_JOYSTICK   = $04;
   cHIDUsageGeneric_GAMEPAD    = $05;
   cHIDUsageGeneric_KEYBOARD   = $06;
   cHIDUsageGeneric_KEYPAD     = $07;
   cHIDUsageGeneric_SYSTEM_CTL = $80;

   cHIDUsageGeneric_X                   = $30;
   cHIDUsageGeneric_Y                   = $31;
   cHIDUsageGeneric_Z                   = $32;
   cHIDUsageGeneric_RX                  = $33;
   cHIDUsageGeneric_RY                  = $34;
   cHIDUsageGeneric_RZ                  = $35;
   cHIDUsageGeneric_SLIDER              = $36;
   cHIDUsageGeneric_DIAL                = $37;
   cHIDUsageGeneric_WHEEL               = $38;
   cHIDUsageGeneric_HATSWITCH           = $39;
   cHIDUsageGeneric_COUNTED_BUFFER      = $3A;
   cHIDUsageGeneric_BYTE_COUNT          = $3B;
   cHIDUsageGeneric_MOTION_WAKEUP       = $3C;
   cHIDUsageGeneric_VX                  = $40;
   cHIDUsageGeneric_VY                  = $41;
   cHIDUsageGeneric_VZ                  = $42;
   cHIDUsageGeneric_VBRX                = $43;
   cHIDUsageGeneric_VBRY                = $44;
   cHIDUsageGeneric_VBRZ                = $45;
   cHIDUsageGeneric_VNO                 = $46;
   cHIDUsageGeneric_SYSCTL_POWER        = $81;
   cHIDUsageGeneric_SYSCTL_SLEEP        = $82;
   cHIDUsageGeneric_SYSCTL_WAKE         = $83;
   cHIDUsageGeneric_SYSCTL_CONTEXT_MENU = $84;
   cHIDUsageGeneric_SYSCTL_MAIN_MENU    = $85;
   cHIDUsageGeneric_SYSCTL_APP_MENU     = $86;
   cHIDUsageGeneric_SYSCTL_HELP_MENU    = $87;
   cHIDUsageGeneric_SYSCTL_MENU_EXIT    = $88;
   cHIDUsageGeneric_SYSCTL_MENU_SELECT  = $89;
   cHIDUsageGeneric_SYSCTL_MENU_RIGHT   = $8A;
   cHIDUsageGeneric_SYSCTL_MENU_LEFT    = $8B;
   cHIDUsageGeneric_SYSCTL_MENU_UP      = $8C;
   cHIDUsageGeneric_SYSCTL_MENU_DOWN    = $8D;

//
// Usages from Simulation Controls Page (0x02)
//

   cHIDUsageSimulation_RUDDER   = $BA;
   cHIDUsageSimulation_THROTTLE = $BB;

//
// Virtual Reality Controls Page (0x03)
//


//
// Sport Controls Page (0x04)
//


//
// Game Controls Page (0x05)
//


//
// Keyboard/Keypad Page (0x07)
//

   // Error "keys"
   cHIDUsageKeyboard_NOEVENT      = $00;
   cHIDUsageKeyboard_ROLLOVER     = $01;
   cHIDUsageKeyboard_POSTFAIL     = $02;
   cHIDUsageKeyboard_UNDEFINED    = $03;

   // Letters
   cHIDUsageKeyboard_aA           = $04;
   cHIDUsageKeyboard_bB           = $05;
   cHIDUsageKeyboard_cC           = $06;
   cHIDUsageKeyboard_dD           = $07;
   cHIDUsageKeyboard_eE           = $08;
   cHIDUsageKeyboard_fF           = $09;
   cHIDUsageKeyboard_gG           = $0A;
   cHIDUsageKeyboard_hH           = $0B;
   cHIDUsageKeyboard_iI           = $0C;
   cHIDUsageKeyboard_jJ           = $0D;
   cHIDUsageKeyboard_kK           = $0E;
   cHIDUsageKeyboard_lL           = $0F;
   cHIDUsageKeyboard_mM           = $10;
   cHIDUsageKeyboard_nN           = $11;
   cHIDUsageKeyboard_oO           = $12;
   cHIDUsageKeyboard_pP           = $13;
   cHIDUsageKeyboard_qQ           = $14;
   cHIDUsageKeyboard_rR           = $15;
   cHIDUsageKeyboard_sS           = $16;
   cHIDUsageKeyboard_tT           = $17;
   cHIDUsageKeyboard_uU           = $18;
   cHIDUsageKeyboard_vV           = $19;
   cHIDUsageKeyboard_wW           = $1A;
   cHIDUsageKeyboard_xX           = $1B;
   cHIDUsageKeyboard_yY           = $1C;
   cHIDUsageKeyboard_zZ           = $1D;

   // Numbers
   cHIDUsageKeyboard_1            = $1E; // or !
   cHIDUsageKeyboard_2            = $1F; // or @
   cHIDUsageKeyboard_3            = $20; // or #
   cHIDUsageKeyboard_4            = $21; // or $
   cHIDUsageKeyboard_5            = $22; // or %
   cHIDUsageKeyboard_6            = $23; // or ^
   cHIDUsageKeyboard_7            = $24; // or &
   cHIDUsageKeyboard_8            = $25; // or *
   cHIDUsageKeyboard_9            = $26; // or (
   cHIDUsageKeyboard_0            = $27; // or )

   cHIDUsageKeyboard_ENTER        = $28; // RETURN is another key
   cHIDUsageKeyboard_ESCAPE       = $29;
   cHIDUsageKeyboard_BACKSPACE    = $2A; // Delete left char
   cHIDUsageKeyboard_TAB          = $2B;
   cHIDUsageKeyboard_SPACE        = $2C;
   cHIDUsageKeyboard_MINUS        = $2D; // or _
   cHIDUsageKeyboard_EQUAL        = $2E; // or +
   cHIDUsageKeyboard_LSQBRACKET   = $2F; // or {
   cHIDUsageKeyboard_RSQBRACKET   = $30; // or }
   cHIDUsageKeyboard_BACKSLASH    = $31; // or |
   cHIDUsageKeyboard_HASHMARK2    = $32; // or ~ Non US Key
   cHIDUsageKeyboard_SEMICOLON    = $33;
   cHIDUsageKeyboard_APOSTROPH    = $34; // or :
   cHIDUsageKeyboard_GRAVEACCENT  = $35; // or Tilde
   cHIDUsageKeyboard_COMMA        = $36; // or <
   cHIDUsageKeyboard_DOT          = $37; // or >
   cHIDUsageKeyboard_SLASH        = $38; // or ?

   cHIDUsageKeyboard_CAPS_LOCK    = $39;

   // Function keys
   cHIDUsageKeyboard_F1           = $3A;
   cHIDUsageKeyboard_F2           = $3B;
   cHIDUsageKeyboard_F3           = $3C;
   cHIDUsageKeyboard_F4           = $3D;
   cHIDUsageKeyboard_F5           = $3E;
   cHIDUsageKeyboard_F6           = $3F;
   cHIDUsageKeyboard_F7           = $40;
   cHIDUsageKeyboard_F8           = $41;
   cHIDUsageKeyboard_F9           = $42;
   cHIDUsageKeyboard_F10          = $43;
   cHIDUsageKeyboard_F11          = $44;
   cHIDUsageKeyboard_F12          = $45;

   cHIDUsageKeyboard_PRINT_SCREEN = $46;
   cHIDUsageKeyboard_SCROLL_LOCK  = $47;

   cHIDUsageKeyboard_PAUSE        = $48;
   cHIDUsageKeyboard_INSERT       = $49;
   cHIDUsageKeyboard_HOME         = $4A;
   cHIDUsageKeyboard_PAGEUP       = $4B;
   cHIDUsageKeyboard_DELETE       = $4C;
   cHIDUsageKeyboard_END          = $4D;
   cHIDUsageKeyboard_PAGEDOWN     = $4E;
   cHIDUsageKeyboard_RIGHT        = $4F;
   cHIDUsageKeyboard_LEFT         = $50;
   cHIDUsageKeyboard_DOWN         = $51;
   cHIDUsageKeyboard_UP           = $52;

   cHIDUsageKeypad_NUM_LOCK       = $53;
   cHIDUsageKeypad_SLASH          = $54;
   cHIDUsageKeypad_STAR           = $55;
   cHIDUsageKeypad_MINUS          = $56;
   cHIDUsageKeypad_PLUS           = $57;
   cHIDUsageKeypad_ENTER          = $58;
   cHIDUsageKeypad_1              = $59;
   cHIDUsageKeypad_2              = $5A;
   cHIDUsageKeypad_3              = $5B;
   cHIDUsageKeypad_4              = $5C;
   cHIDUsageKeypad_5              = $5D;
   cHIDUsageKeypad_6              = $5E;
   cHIDUsageKeypad_7              = $5F;
   cHIDUsageKeypad_8              = $60;
   cHIDUsageKeypad_9              = $61;
   cHIDUsageKeypad_0              = $62;
   cHIDUsageKeypad_DOT            = $63;

   cHIDUsageKeyboard_BACKSLASH2   = $64; // or | Non US key

   cHIDUsageKeyboard_APPLICATION  = $65;

   // Keys not for Windows
   cHIDUsageKeyboard_POWER        = $66;
   cHIDUsageKeypad_EQUAL2         = $67;

   // Keys not for Windows
   cHIDUsageKeyboard_F13          = $68;
   cHIDUsageKeyboard_F14          = $69;
   cHIDUsageKeyboard_F15          = $6A;
   cHIDUsageKeyboard_F16          = $6B;
   cHIDUsageKeyboard_F17          = $6C;
   cHIDUsageKeyboard_F18          = $6D;
   cHIDUsageKeyboard_F19          = $6E;
   cHIDUsageKeyboard_F20          = $6F;
   cHIDUsageKeyboard_F21          = $70;
   cHIDUsageKeyboard_F22          = $71;
   cHIDUsageKeyboard_F23          = $72;
   cHIDUsageKeyboard_F24          = $73;
   cHIDUsageKeyboard_EXECUTE      = $74;
   cHIDUsageKeyboard_HELP         = $75;
   cHIDUsageKeyboard_MENU         = $76;
   cHIDUsageKeyboard_SELECT       = $77;
   cHIDUsageKeyboard_STOP         = $78;
   cHIDUsageKeyboard_AGAIN        = $79;
   cHIDUsageKeyboard_UNDO         = $7A;
   cHIDUsageKeyboard_CUT          = $7B;
   cHIDUsageKeyboard_COPY         = $7C;
   cHIDUsageKeyboard_PASTE        = $7D;
   cHIDUsageKeyboard_FIND         = $7E;
   cHIDUsageKeyboard_MUTE         = $7F;
   cHIDUsageKeyboard_VOLUME_UP    = $80;
   cHIDUsageKeyboard_VOLUME_DOWN  = $81;

   cHIDUsageKeyboard_LOCKCAPS     = $82;
   cHIDUsageKeyboard_LOCKNUM      = $83;
   cHIDUsageKeyboard_LOCKSCROLL   = $84;

   cHIDUsageKeypad_COMMA          = $85;
   cHIDUsageKeypad_EQUALSIGN      = $86;

   cHIDUsageKeyboard_INATL1       = $87;
   cHIDUsageKeyboard_INATL2       = $88;
   cHIDUsageKeyboard_INATL3       = $89;
   cHIDUsageKeyboard_INATL4       = $8A;
   cHIDUsageKeyboard_INATL5       = $8B;
   cHIDUsageKeyboard_INATL6       = $8C;
   cHIDUsageKeyboard_INATL7       = $8D;
   cHIDUsageKeyboard_INATL8       = $8E;
   cHIDUsageKeyboard_INATL9       = $8F;
   cHIDUsageKeyboard_LANG1        = $90;
   cHIDUsageKeyboard_LANG2        = $91;
   cHIDUsageKeyboard_LANG3        = $92;
   cHIDUsageKeyboard_LANG4        = $93;
   cHIDUsageKeyboard_LANG5        = $94;
   cHIDUsageKeyboard_LANG6        = $95;
   cHIDUsageKeyboard_LANG7        = $96;
   cHIDUsageKeyboard_LANG8        = $97;
   cHIDUsageKeyboard_LANG9        = $98;

   cHIDUsageKeyboard_ALTERASE     = $99;
   cHIDUsageKeyboard_SYSREQ       = $9A;
   cHIDUsageKeyboard_CANCEL       = $9B;
   cHIDUsageKeyboard_CLEAR        = $9C;
   cHIDUsageKeyboard_PRIOR        = $9D;
   cHIDUsageKeyboard_RETURN       = $9E;
   cHIDUsageKeyboard_SEPARATOR    = $9F;
   cHIDUsageKeyboard_OUT          = $A0;
   cHIDUsageKeyboard_OPER         = $A1;
   cHIDUsageKeyboard_CLEAR_AGAIN  = $A2;
   cHIDUsageKeyboard_CRSEL        = $A3;
   cHIDUsageKeyboard_EXSEL        = $A4;

   cHIDUsageKeyboard_LCTRL        = $E0;
   cHIDUsageKeyboard_LSHFT        = $E1;
   cHIDUsageKeyboard_LALT         = $E2;
   cHIDUsageKeyboard_LGUI         = $E3;
   cHIDUsageKeyboard_RCTRL        = $E4;
   cHIDUsageKeyboard_RSHFT        = $E5;
   cHIDUsageKeyboard_RALT         = $E6;
   cHIDUsageKeyboard_RGUI         = $E7;

// and hundreds more...

//
// LED Page (0x08)
//

   cHIDUsageLED_NUM_LOCK             = $01;
   cHIDUsageLED_CAPS_LOCK            = $02;
   cHIDUsageLED_SCROLL_LOCK          = $03;
   cHIDUsageLED_COMPOSE              = $04;
   cHIDUsageLED_KANA                 = $05;
   cHIDUsageLED_POWER                = $06;
   cHIDUsageLED_SHIFT                = $07;
   cHIDUsageLED_DO_NOT_DISTURB       = $08;
   cHIDUsageLED_MUTE                 = $09;
   cHIDUsageLED_TONE_ENABLE          = $0A;
   cHIDUsageLED_HIGH_CUT_FILTER      = $0B;
   cHIDUsageLED_LOW_CUT_FILTER       = $0C;
   cHIDUsageLED_EQUALIZER_ENABLE     = $0D;
   cHIDUsageLED_SOUND_FIELD_ON       = $0E;
   cHIDUsageLED_SURROUND_FIELD_ON    = $0F;
   cHIDUsageLED_REPEAT               = $10;
   cHIDUsageLED_STEREO               = $11;
   cHIDUsageLED_SAMPLING_RATE_DETECT = $12;
   cHIDUsageLED_SPINNING             = $13;
   cHIDUsageLED_CAV                  = $14;
   cHIDUsageLED_CLV                  = $15;
   cHIDUsageLED_RECORDING_FORMAT_DET = $16;
   cHIDUsageLED_OFF_HOOK             = $17;
   cHIDUsageLED_RING                 = $18;
   cHIDUsageLED_MESSAGE_WAITING      = $19;
   cHIDUsageLED_DATA_MODE            = $1A;
   cHIDUsageLED_BATTERY_OPERATION    = $1B;
   cHIDUsageLED_BATTERY_OK           = $1C;
   cHIDUsageLED_BATTERY_LOW          = $1D;
   cHIDUsageLED_SPEAKER              = $1E;
   cHIDUsageLED_HEAD_SET             = $1F;
   cHIDUsageLED_HOLD                 = $20;
   cHIDUsageLED_MICROPHONE           = $21;
   cHIDUsageLED_COVERAGE             = $22;
   cHIDUsageLED_NIGHT_MODE           = $23;
   cHIDUsageLED_SEND_CALLS           = $24;
   cHIDUsageLED_CALL_PICKUP          = $25;
   cHIDUsageLED_CONFERENCE           = $26;
   cHIDUsageLED_STAND_BY             = $27;
   cHIDUsageLED_CAMERA_ON            = $28;
   cHIDUsageLED_CAMERA_OFF           = $29;
   cHIDUsageLED_ON_LINE              = $2A;
   cHIDUsageLED_OFF_LINE             = $2B;
   cHIDUsageLED_BUSY                 = $2C;
   cHIDUsageLED_READY                = $2D;
   cHIDUsageLED_PAPER_OUT            = $2E;
   cHIDUsageLED_PAPER_JAM            = $2F;
   cHIDUsageLED_REMOTE               = $30;
   cHIDUsageLED_FORWARD              = $31;
   cHIDUsageLED_REVERSE              = $32;
   cHIDUsageLED_STOP                 = $33;
   cHIDUsageLED_REWIND               = $34;
   cHIDUsageLED_FAST_FORWARD         = $35;
   cHIDUsageLED_PLAY                 = $36;
   cHIDUsageLED_PAUSE                = $37;
   cHIDUsageLED_RECORD               = $38;
   cHIDUsageLED_ERROR                = $39;
   cHIDUsageLED_SELECTED_INDICATOR   = $3A;
   cHIDUsageLED_IN_USE_INDICATOR     = $3B;
   cHIDUsageLED_MULTI_MODE_INDICATOR = $3C;
   cHIDUsageLED_INDICATOR_ON         = $3D;
   cHIDUsageLED_INDICATOR_FLASH      = $3E;
   cHIDUsageLED_INDICATOR_SLOW_BLINK = $3F;
   cHIDUsageLED_INDICATOR_FAST_BLINK = $40;
   cHIDUsageLED_INDICATOR_OFF        = $41;
   cHIDUsageLED_FLASH_ON_TIME        = $42;
   cHIDUsageLED_SLOW_BLINK_ON_TIME   = $43;
   cHIDUsageLED_SLOW_BLINK_OFF_TIME  = $44;
   cHIDUsageLED_FAST_BLINK_ON_TIME   = $45;
   cHIDUsageLED_FAST_BLINK_OFF_TIME  = $46;
   cHIDUsageLED_INDICATOR_COLOR      = $47;
   cHIDUsageLED_RED                  = $48;
   cHIDUsageLED_GREEN                = $49;
   cHIDUsageLED_AMBER                = $4A;
   cHIDUsageLED_GENERIC_INDICATOR    = $3B;

//
//  Button Page (0x09)
//
//  There is no need to label these usages.
//


//
//  Ordinal Page (0x0A)
//
//  There is no need to label these usages.
//


//
//  Telephony Device Page (0x0B)
//

   cHIDUsageTelephony_PHONE               = $01;
   cHIDUsageTelephony_ANSWERING_MACHINE   = $02;
   cHIDUsageTelephony_MESSAGE_CONTROLS    = $03;
   cHIDUsageTelephony_HANDSET             = $04;
   cHIDUsageTelephony_HEADSET             = $05;
   cHIDUsageTelephony_KEYPAD              = $06;
   cHIDUsageTelephony_PROGRAMMABLE_BUTTON = $07;

//
// and others...
//

type
   TUsage  = Word;
   PTUsage = ^TUsage;

implementation

end.
