// SetupAPI contains the consts, types and prototypes of all the
// HID related SetupDi* functions mentioned but not documented in
// the 98 DDK
// the comments are mainly from SetupAPI.h since there is no other info

unit SetupAPI;

interface

uses
   Windows; // for DWORD etc.

const
   // internal constant to name the DLL in prototypes
   SetupAPIDll = 'SetupAPI.dll';

   // Flags for TSPDeviceInterfaceData.Flags field.
   cSPINTActive  = $00000001;
   cSPINTDefault = $00000002;
   cSPINTRemoved = $00000004;

   // Flags controlling what is included in the device information set
   // built by SetupDiGetClassDevs
   cDIGCFDefault         = $00000001;  // only valid with DIGCF_DEVICEINTERFACE
   cDIGCFPresent         = $00000002;
   cDIGCFAllClasses      = $00000004;
   cDIGCFProfile         = $00000008;
   cDIGCFDeviceInterface = $00000010;

type
   // Device information structure (references a device instance
   // that is a member of a device information set)
   TSPDevInfoData = packed record
      cbSize:     DWORD;
      IClassGuid: TGUID;
      DevInst:    DWORD;
      Reserved:   DWORD;
   end;

   PTSPDevInfoData = ^TSPDevInfoData;

   // Device interface information structure (references a device
   // interface that is associated with the device information
   // element that owns it).
   TSPDeviceInterfaceData = packed record
      cbSize:             DWORD;  // size of this structure
      InterfaceClassGuid: TGUID;
      Flags:              DWORD;
      Reserved:           DWORD;
   end;

   // this structure is a simplification
   // alternatively allocate memory directly
   // and hand it in with cbSize set accordingly
   TSPDeviceInterfaceDetailData = packed record
      cbSize:     DWORD;
      DevicePath: array [0..255] of Char;
   end;

   // we always use the ASCII variant of the functions (suffix A)
   function SetupDiGetClassDevs
     (
      var Guid:       TGUID;
          Enumerator: PChar;
          hWin:       HWND;
          Flags:      DWORD
     ): THandle; stdcall; external SetupAPIDll name 'SetupDiGetClassDevsA';

   function SetupDiEnumDeviceInterfaces
     (
          DeviceInfoSet:       THandle;
          DeviceInfoData:      PTSPDevInfoData;
      var InterfaceClassGuid:  TGUID;
          MemberIndex:         DWORD;
      var DeviceInterfaceData: TSPDeviceInterfaceData
     ): DWORD; stdcall; external SetupAPIDll name 'SetupDiEnumDeviceInterfaces';

   function SetupDiGetDeviceInterfaceDetail
     (
          DeviceInfoSet:                 THandle;
      var DeviceInterfaceData:           TSPDeviceInterfaceData;
      var DeviceInterfaceDetailData:     TSPDeviceInterfaceDetailData;
          DeviceInterfaceDetailDataSize: DWORD;
      var RequiredSize:                  DWORD;
          DeviceInfoData:                PTSPDevInfoData
     ): DWORD; stdcall; external SetupAPIDll name 'SetupDiGetDeviceInterfaceDetailA';

implementation

end.
